/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        03.11.2007
 Description:    Webserver Config-File

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CONFIG_H_
	#define _CONFIG_H_	

	#define AVR_WEBSERVER_NAME "Uwe's LED-Cube"
	#define AVR_WEBSERVER_VERSION "1.0"
	
	//ETH_M32_EX (www.ulrichradig.de)
	#define USE_ENC28J60	1
	//Holger Buss (www.mikrocontroller.com) Mega32-Board
	#define USE_RTL8019	0
	
	//Konfiguration der PORTS (HEX)
	//1=OUTPUT / 0=INPUT
	#define OUTA 		0x00
	#ifdef POLLIN_NETIO
		#define PULLUPA		0xe0  // PullUps on PA5..PA7
	#else
		#define PULLUPA		0xf0  // PullUps on PA4..PA7
	#endif
	#define OUTC 		0xff
	#define OUTD 		0x00
	//Achtung!!!! an PORTB ist der ENC
	//nur ndern wenn man wei was man macht!


	//Umrechnung von IP zu unsigned long
	#define IP(a,b,c,d) ((unsigned long)(d)<<24)+((unsigned long)(c)<<16)+((unsigned long)(b)<<8)+a

	//IP des Webservers
	#define MYIP		IP(192,168,1,10)

	//Netzwerkmaske
	#define NETMASK		IP(255,255,255,0)
	
	//IP des Routers
	#define ROUTER_IP	IP(192,168,1,254)
	
	//IP des NTP-Servers z.B. Server 0.de.pool.ntp.org
	#define USE_NTP		1 //1 = NTP Client on
	#define NTP_IP		IP(192,168,1,254)
	
	//Broadcast-Adresse fr WOL
	#define USE_WOL		0 //1 = WOL on
	#define WOL_BCAST_IP	IP(192,168,1,255)
	#define WOL_MAC 	{0x02,0x04,0x25,0x18,0xb7,0x3e}
	
	//MAC Adresse des Webservers (00:04:25 Atmel Inc.)
	#define MYMAC1	0x02
	#define MYMAC2	0x04
	#define MYMAC3	0x25
	#define MYMAC4  0x0e
	#define MYMAC5  0x3a
	#define MYMAC6  0x23

	// use DHCP  ***experimental***
	#define USE_DHCP	0
	
	//Taktfrequenz
	#ifndef F_CPU
	#define F_CPU 11059200UL
	#endif
	
	//Timertakt intern oder extern
	#define EXTCLOCK 0 //0=Intern 1=Externer Uhrenquarz

	//Baudrate der seriellen Schnittstelle
	#define BAUDRATE 9600

	// Onewire support
	#define USE_ONEWIRE		1
    #define ONEWIRE_DS1820		1
    #define ONEWIRE_DS1822		1
	#ifdef POLLIN_NETIO
		#define W1_PIN  PA4
		#define W1_IN   PINA
		#define W1_OUT  PORTA
		#define W1_DDR  DDRA
	#else
		#define W1_PIN  PD7
		#define W1_IN   PIND
		#define W1_OUT  PORTD
		#define W1_DDR  DDRD
	#endif
	
	//LCD Routinen mit einbinden
	#define USE_SER_LCD		0
	//LCD im 4Bit Mode oder seriell
	#define USE_LCD_4Bit		0

	//AD-Wandler benutzen?
	#define USE_ADC			0

	//Kamera mit einbinden
	//Kamera arbeitet nur mit einem 14,7456Mhz Quarz!
	#define USE_CAM			0
	#define USE_SERVO		0
	//In cam.c knnen weitere Parameter eingestellt werde
	//z.B. Licht, Kompression usw.
	//Auflsungen
	//0 = 160x120 Pixel krzer (zum testen OK ;-)
	//1 = 320x240 Pixel ca. 10 Sek. bei einem Mega644
	//2 = 640x480 Pixel lnger (dauert zu lang!)
	#define CAM_RESELUTION		0
	
	// Webserver einbinden?
	#define USE_HTTPD			1
	//Webserver mit Passwort? (0 == mit Passwort)
	#define HTTP_AUTH_DEFAULT	1
	//AUTH String "USERNAME:PASSWORT" max 14Zeichen 
	//fr Username:Passwort
	#define HTTP_AUTH_STRING "bgr:dvmxzu"
	
	//Email vesand benutzen? Konfiguration des
	//Emailclient in der Sendmail.h
	#define USE_MAIL        0
    
	//Empfang von Wetterdaten auf der Console (ber HTTP_GET)
	#define GET_WEATHER     0
    
	//Commandos und Ausgaben erfolgen ber Telnet
	#define CMD_TELNET  0
	
	// Erweiterung fuer LED-Cube
	#define LED_CUBE    1
    
#endif //_CONFIG_H


