/*----------------------------------------------------------------------------
 Copyright:      Uwe Berger; bergeruw@gmx.net
 Author:         Uwe Berger
 Remarks:
 known Problems: none
 Version:        17.03.2010
 Description:    Routinen fuer LED-Cube

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "led_cube.h"
#include "led_cube_intro.h"

#if LED_CUBE
volatile uint8_t led[CUBE_XY];


//static const char out_set_xyz[] = OUT_SET_XYZ;
//static const char out_update_ok[] = OUT_UPDATE_OK;

CUBE_STATUS cube_status;

uint8_t spalte = 0;
uint8_t ebene  = 0;
uint8_t intro_index = 0;

volatile uint8_t cube_connections = 0;
volatile uint16_t intro_timer = 0;

// (interne) Prototypen
void respond_P(const char *, unsigned char);
void _respond(uint16_t, unsigned char);
void clear_cube_status(void);


//----------------------------------------------------------------------------
void led_cube_init (void)
{
	/* Timer und Interrupt initialisieren*/
	TCNT0   =  T0_PRELOAD;			/* Preload-Wert     */
	TCCR0  |=  (1<<CS01)|(1<<CS00); /* Vorteiler 1:64 	*/
	TIMSK  |= (1<<TOIE0);			/* Interrupt Timer0 */
	
	// IP-Port...
	add_tcp_app (CUBE_PORT, (void(*)(unsigned char))cube_data);
}

//----------------------------------------------------------------------------
void cube_data(unsigned char index)
{
	uint16_t i;
	uint8_t temp;

	// eine neue Verbindung wurde aufgebaut
	if (tcp_entry[index].app_status <= 1) {
		tcp_entry[index].app_status = 1; 
		cube_connections++;
		cube_status.is_cmd  = 0;
		cube_status.is_arg  = 0; 
		cube_status.cmd[0]  = 0;
		cube_status.cmd_idx = 0;
		cube_status.arg_len = 0;
		// und schon mal die Diemension dem Client verraten...
		respond_P(PSTR(OUT_SET_XYZ),index);
	}	

    // eine Verbindung wurde abgebaut
    if (tcp_entry[index].status & FIN_FLAG) {
		cube_connections--;
        return;
    }
	
	// Ist das Paket fr die Anwendung bestimmt ?
	if (tcp_entry[index].status&PSH_FLAG) {		
		tcp_entry[index].app_status = 2;
		
		// Daten analysieren
		for (i=TCP_DATA_START_VAR; i<(TCP_DATA_END_VAR); i++) {

			// Zeilenende (das war es!)
			if ((eth_buffer[i] == '\r') || (eth_buffer[i] == '\n')) {
				// ------------------------------
				// Auswertung fuer CMDs ohne ARGs
				// ------------------------------
				// Kommando get_xyz				
				if(strcasestr_P((char*)&cube_status.cmd[0],PSTR("GET_XYZ"))) {
					respond_P(PSTR(OUT_SET_XYZ),index);
					clear_cube_status();
					return;
				}
				// auch fuer ungueltige Kommandos Status zuruecksetzen
				clear_cube_status();
			}
			
			// Leerzeichen
			if (eth_buffer[i] == ' ') {
				// wenn bereits cmd, dann kommt ab jetzt das arg
				if (cube_status.is_cmd) {
					cube_status.is_cmd = 0;
					// -----------------------------------
					// Auswertung fuer CMDs mit Argumenten
					// -----------------------------------
					// Kommando set_led
					if(strcasestr_P((char*)&cube_status.cmd[0],PSTR("SET_LED"))) {
						cube_status.is_arg = 1;
					}
				}
			}
				
			// alle Zeichen ab Whitespace
			if (eth_buffer[i] > ' ') {
				// wenn noch kein CMD oder ARG, dann kann es nur ein neues CMD sein
				if (!cube_status.is_cmd && !cube_status.is_arg) cube_status.is_cmd=1;
				// CMD in Puffer uebernehmen
				if (cube_status.is_cmd && (cube_status.cmd_idx < CMD_BUFFER_LEN)) {
					cube_status.cmd[cube_status.cmd_idx++]=eth_buffer[i];
					cube_status.cmd[cube_status.cmd_idx]=0;
				}
				
				// Argument uebernehmen
				if (cube_status.is_arg) {
					// LED-Belegung kommt in Hex an!
					if (cube_status.arg_len < (CUBE_XY*2)) {
						// uppercase
						if (eth_buffer[i]>70) eth_buffer[i] -= 32;
						// Hex2Int ueber zwei Stellen von eth_buffer
						if (eth_buffer[i]>57) {
							temp = eth_buffer[i] - 55; 
						} else {
							temp = eth_buffer[i] - 48;
						}
						if (!(cube_status.arg_len%2)) {
							led[cube_status.arg_len/2] = 16 * temp;
						} else { 
							led[cube_status.arg_len/2] = led[cube_status.arg_len/2] + temp;
						}
						cube_status.arg_len++;	
						// theoretisches Ende erreicht? dann hier auch abbrechen
						if (cube_status.arg_len == (CUBE_XY*2)) {
							respond_P(PSTR(OUT_UPDATE_OK),index);
							clear_cube_status();
						}
					} else {
						respond_P(PSTR(OUT_UPDATE_OK),index);
						clear_cube_status();
						return;
					}
				}
			
			}
				
		}
		
		// empfangenes Paket besttigen
		tcp_entry[index].status =  ACK_FLAG;	
		create_new_tcp_packet(0,index);
		return;
	}
	
	// ist dies ein ACK von einem gesendeten Paket
	if (tcp_entry[index].status&ACK_FLAG) {
		tcp_entry[index].time = TCP_TIME_OFF;
		return;
	}

	// Timeout kein ack angekommen
	if (tcp_entry[index].status == 0) {
		return;
	}

}

//----------------------------------------------------------------------------
void respond_P(const char *pstr, unsigned char index)
{
	uint16_t len = strlen_P(pstr);
	memcpy_P(&eth_buffer[TCP_DATA_START],pstr,len);
	_respond(len, index);
}

//----------------------------------------------------------------------------
void _respond(uint16_t len, unsigned char index)
{
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	tcp_entry[index].time = TCP_TIME_OFF;
	create_new_tcp_packet(len,index);
}

//----------------------------------------------------------------------------
void clear_cube_status(void)
{
	cube_status.is_cmd  = 0;
	cube_status.cmd_idx = 0;
	cube_status.is_arg  = 0; 
	cube_status.cmd[0]  = 0;
	cube_status.arg_len = 0;
}

//----------------------------------------------------------------------------
void command_cube_status (void)
{
	uint8_t i, j;
	
	usart_write(CUBE_DIMENSION);
	usart_write(CUBE_IP_PORT);
	usart_write("Connects.: %1i\r\n", cube_connections);
	for (j=0; j<CUBE_X; j++) {
		for (i=0; i<CUBE_Y; i++) {
			usart_write("%2x ", led[(j*CUBE_X)+i]);
		}
		usart_write("\r\n");
	}
	usart_write("\r\n");
}


//----------------------------------------------------------------------------
// Interruptroutine Steuerung Cube-Hardware
ISR(TIMER0_OVF_vect)
{
	
#if 1	
//	uint8_t mask=1;
	uint8_t mask=128;

	/* Preload-Wert des Timers neu setzen */
	TCNT0   =  T0_PRELOAD;
	
	/*************************************************************************/
	/* vorhergehende Zeile sauber auf LOW schalten, wegen "LED-Nachleuchten" */
	/*************************************************************************/
		/* dazu Schieberegisterkette loeschen (Low-Impulse auf SCL) */
	PORT_SR &= ~(1 << PIN_SCL);	/* SCL auf 0 */
	PORT_SR |=  (1 << PIN_SCL);	/* SCL auf 1 */
		/* 0-1-Flanke an RCK bringt die Daten aus den Registern in die Latches */
	PORT_SR &= ~(1 << PIN_RCK);	/* RCK auf 0 */
	PORT_SR |=  (1 << PIN_RCK);	/* RCK auf 1 */
	
   	/***********************************/
	/* naechste Zeile ermitteln/ausgeben */
   	/***********************************/
	/* Ebenen und Spalten weiterzaehlen */
	if (spalte < CUBE_Y - 1) {
		spalte++;
	} else {
		if (ebene  < CUBE_Z - 1) {ebene++;}  else {ebene = 0;}
		spalte = 0;
	}

	/* Zeile ins Schiebergister schieben */
	for(uint8_t i=0; i<CUBE_X; ++i) {
		if (led[(ebene*CUBE_Z) + spalte] & mask)
			PORT_SR |=  (1 << PIN_SER); /* 1 */
		else
			PORT_SR &= ~(1 << PIN_SER); /* 0 */
//		mask *=2;
		mask /=2;
		/* 0-1-Flanke an SCK schiebt Daten in den Registern um eine Stelle weiter */
		PORT_SR &= ~(1 << PIN_SCK); /* SCK auf 0 */
		PORT_SR |=  (1 << PIN_SCK); /* SCK auf 1 */
	}
	
	/* Ebene ins Schieberegister schieben */
	for (uint8_t i = 0; i < CUBE_Z; i++) {
		if (i != (CUBE_Z - ebene - 1)) {
			PORT_SR &= ~(1 << PIN_SER);	/* 0 */
		} else {
			PORT_SR |=  (1 << PIN_SER); /* 1 */		
		}
		/* 0-1-Flanke an SCK schiebt Daten in den Registern um eine Stelle weiter */
		PORT_SR &= ~(1 << PIN_SCK); /* SCK auf 0 */
		PORT_SR |=  (1 << PIN_SCK); /* SCK auf 1 */
	}
	
	/* Spalte ins Schieberegister schieben */
	for (uint8_t i = 0; i < CUBE_Y; i++) {
		if (i != (CUBE_Y - spalte - 1)) {
			PORT_SR &= ~(1 << PIN_SER); /* 0 */
		} else {
			PORT_SR |=  (1 << PIN_SER); /* 1 */		
		}	
		/* 0-1-Flanke an SCK schiebt Daten in den Registern um eine Stelle weiter */
		PORT_SR &= ~(1 << PIN_SCK); /* SCK auf 0 */
		PORT_SR |=  (1 << PIN_SCK); /* SCK auf 1 */
	}
	
	/* 0-1-Flanke an RCK bringt die Daten aus den Registern in die Latches */
	PORT_SR &= ~(1 << PIN_RCK);	/* RCK auf 0 */
	PORT_SR |=  (1 << PIN_RCK);	/* RCK auf 1 */
#endif

// war mal ein gut gemeinter Versuch, geht aber nicht 100%ig...
#if 0
	uint32_t sbuf=0;

	TCNT0   =  T0_PRELOAD;
	
	/* Ebenen und Spalten weiterzaehlen */
	spalte++;
	if (spalte > (CUBE_Y-1)) {
		spalte=0;
		ebene++;
		if (ebene > (CUBE_Z-1))	ebene=0;
	}
	
	// sbuf zusammensetzen (led-ebene-spalte)
	// ...
	// 2^3 = 1 << 3 = 8 0b0000 1000
	// ... hier scheint es noch Optimierungspotential zu geben...
	//
	sbuf = led[(ebene*CUBE_Z)+spalte]*0x10000+(2^ebene)*0x100+(2^spalte);
	// ... oder so besser?
	//sbuf = (led[(ebene*CUBE_Z)+spalte] << 16)|((1<<ebene)<<8)|(1<<spalte);
	// ... oder so?
	//sbuf = led[(ebene*CUBE_Z)+spalte]*0x10000+(1<<ebene)*0x100+(1<<spalte);
	// ... oder so?
	//sbuf = (led[(ebene*CUBE_Z)+spalte]<<16)+((1<<ebene)<<8)+(1<<spalte);
	
	// dazu Schieberegisterkette loeschen (Low-Impulse auf SCL)
	PORT_SR &= ~(1 << PIN_SCL);	/* SCL auf 0 */
	PORT_SR |=  (1 << PIN_SCL);	/* SCL auf 1 */
	// 0-1-Flanke an RCK bringt die Daten aus den Registern in die Latches
	PORT_SR &= ~(1 << PIN_RCK);	/* RCK auf 0 */
	PORT_SR |=  (1 << PIN_RCK);	/* RCK auf 1 */
	
	// Inhalt von sbuf rausschieben 
	// ... ist erstmal auf 8x8x8 "optimiert"...
	//
	for (uint8_t i=0; i<24; i++) {
		PORT_SR &= ~(1 << PIN_SER); 						// 0
		if (sbuf & 0x800000) PORT_SR |=  (1 << PIN_SER);	// 1
		sbuf <<=1;
		/* 0-1-Flanke an SCK schiebt Daten in den Registern um eine Stelle weiter */
		PORT_SR &= ~(1 << PIN_SCK); /* SCK auf 0 */
		PORT_SR |=  (1 << PIN_SCK); /* SCK auf 1 */
	}

	// 0-1-Flanke an RCK bringt die Daten aus den Registern in die Latches 
	PORT_SR &= ~(1 << PIN_RCK);	/* RCK auf 0 */
	PORT_SR |=  (1 << PIN_RCK);	/* RCK auf 1 */
#endif

	intro_timer++;
}

//----------------------------------------------------------------------------
void led_cube_intro (void) {
	for (uint8_t i=0; i < CUBE_XY; i++) {
		led[i]=pgm_read_byte(&intro[intro_index*CUBE_XY+i]);
	}
	intro_index++;
	if (!(intro_index<INTRO_MAX)) intro_index=0;
}



#endif //LED_CUBE


