/*----------------------------------------------------------------------------
 Copyright:      Uwe Berger (bergeruw@gmx.net)
 Author:         Uwe Berger
 Remarks:        
 known Problems: none
 Version:        17.03.2010
 Description:    Routinen fuer LED-Cube

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include <config.h>

#ifndef _LED_CUBE_H
	#define _LED_CUBE_H

	#include <avr/io.h>
	#include <avr/pgmspace.h>
	#include "stack.h"
	//#include "led_cube_intro.h"
	
	#define CUBE_DEBUG usart_write
	
	
	/*
	Hardwarekonfiguration:
	----------------------
	Registerreihenfolge (von SER gesehen):
	(Spalten/Ebenen nochmal prfen)
	1: Spalten
	2: Ebenen
	3: Zeilen
	PC4 -> SER \
	PC5 -> SCK   \ 8Bit-Schieberegister 
	PC6 -> RCK   / (74xx595x; G auf L)
	PC7 -> SCL /
	*/
	#define PORT_SR PORTC
	#define PIN_SER	PC4
	#define PIN_SCK PC5
	#define PIN_RCK PC6
	#define PIN_SCL PC7

	// Portadresse der Cube-Anwendung
	#define CUBE_PORT 12345

	// Cube-Dimensionen
	#define CUBE_X 8
	#define CUBE_Y 8
	#define CUBE_Z 8
	#define CUBE_XY (CUBE_X * CUBE_Y)
	#define CUBE_XYZ (CUBE_X * CUBE_Y * CUBE_Z)
	
	
	/* Timer-Werte berechnen */
	#define CUBE_REFRESH    50		/* Bildwiederholfrequenz in Hz */
	#define T0_PRESCALE     64		/* Vorteiler des Timers        */
	#define T0_MAX_COUNTER  255		/* Maximalwert des Timers      */
	#ifdef SINGLE_LED
		#define T0_FREQ  (CUBE_REFRESH * CUBE_Y * CUBE_Z * CUBE_X)	/* Zeilenfrequenz */
	#else
		#define T0_FREQ  (CUBE_REFRESH * CUBE_Y * CUBE_Z)			/* Zeilenfrequenz */
	#endif
	#define T0_PRELOAD  (T0_MAX_COUNTER-(F_CPU/T0_PRESCALE/T0_FREQ)) /* result. Preloadwert  */
	
	/* Antwort auf Befehl GET_XYZ */
	#define TOSTR(val) STR(val)
	#define STR(val) #val
	#define OUT_SET_XYZ "set_xyz " TOSTR(CUBE_X) " " TOSTR(CUBE_Y) " " TOSTR(CUBE_Z) "\r\n"

	/* Antwort auf Befehl SET_LED */
	#define OUT_UPDATE_OK "update_ok\r\n"

	// weitere feste Antworten auf diverse Kommandos...
	#define CUBE_DIMENSION "Dimension: " TOSTR(CUBE_X) " " TOSTR(CUBE_Y) " " TOSTR(CUBE_Z) "\r\n"
	#define CUBE_DIMENSION_WEB TOSTR(CUBE_X) "x" TOSTR(CUBE_Y) "x" TOSTR(CUBE_Z)
	
	#define CUBE_IP_PORT   "IP-Port..: " TOSTR(CUBE_PORT) "\r\n"
	#define CUBE_IP_PORT_WEB   TOSTR(CUBE_PORT)

	#define INTRO_FREQ			3		// Frequenz der einzelnen Animationsbilder
	#define INTRO_DELAY_TIME 	T0_FREQ / INTRO_FREQ

	#define CMD_BUFFER_LEN 8

	typedef struct {
		uint8_t is_cmd :1;
		uint8_t is_arg :1;
		uint8_t cmd_idx;
		unsigned char cmd[CMD_BUFFER_LEN+1];
		uint16_t arg_len;
	} CUBE_STATUS;


	volatile uint8_t led[CUBE_XY];
	volatile uint8_t cube_connections;
	volatile uint16_t intro_timer;
		
	void led_cube_init (void);
	void cube_data (unsigned char);
	void command_cube_status (void);
	void led_cube_intro (void);
	
 
#endif //_LED_CUBE_H

