#*************************************************************************
# Ein Client fuer cube_viewer.tcl...
#
# Punkte und Linien zeichnen....
#


set gvar(host) "localhost"
set gvar(port) 12345

set gvar(led_count) 0

#*************************************************************************
# Routinen
#
#
#----- receice -----
#
# Wird aufgerufen, wenn etwas im Empfangspuffer drin ist...
#
proc receive {sock} {
	if {[eof $sock] || [catch {gets $sock line}]} {
		close $sock
	} else {
		if {$line != ""} {
			eval $line
		}
	}
}


#----- set_xyz -----
#   (Serverbefehl)
#
proc set_xyz {x y z} {
	global gvar
	
	set gvar(x) $x
	set gvar(y) $y
	set gvar(z) $z
	set gvar(led_count) [expr $gvar(x) * $gvar(y) * $gvar(z)]
	init_led
}


#----- update_ok -----
#   (Serverbefehl)
#
proc update_ok {} {
	global gvar
#	after 100 [list led_test $gvar(ebene_neu)]

}


#----- init_led -----
#
proc init_led {} {
	global gvar
	set gvar(led) {}
	for {set i 0} {$i < $gvar(led_count)} {incr i} {set gvar(led) [linsert $gvar(led) end 0]}
}


#----- send_cmd -----
#
proc send_cmd {cmd} {
	global gvar
	puts $gvar(sock) $cmd                          
	flush $gvar(sock) 
}


#---- draw_point ----
#
proc draw_point {x y z} {
	global gvar
	set i [expr $x + ($y * $gvar(x)) + ($z * $gvar(x) * $gvar(y))]
	set gvar(led) [lreplace $gvar(led) $i $i 1]
}


#---- get_distance ----
#
proc get_distance {x1 y1 z1 x2 y2 z2} {
	global gvar
	set dist [expr sqrt([expr pow([expr $x2 - $x1], 2)\
                                + pow([expr $y2 - $y1], 2)\
			        + pow([expr $z2 - $z1], 2)])]
	return $dist
}


#---- compute_center ----
#
proc compute_center {x1 y1 z1 x2 y2 z2} {
	set mx [expr ($x1 + $x2)/2]
	set my [expr ($y1 + $y2)/2]
	set mz [expr ($z1 + $z2)/2]

	set mxr [expr ($x1 + $x2)/2.0]
	set myr [expr ($y1 + $y2)/2.0]
	set mzr [expr ($z1 + $z2)/2.0]

	if {([expr $mx - $mxr] == 0.0) && ([expr $my - $myr] == 0.0) && ([expr $mz - $mzr] == 0.0)} {
		draw_point $mx $my $mz
		puts "--> $mxr $myr $mzr"
	} else {
		puts "<-- $mxr $myr $mzr"
	}

	set ret [list $mx $my $mz]
	return $ret
}


#---- draw_line ----
#
proc draw_line_centers {xa ya za xe ye ze} {
	global gvar
	
	foreach {mx my mz} [compute_center $xa $xa $za $xe $ye $ze] {break}
	
	if {[get_distance $xa $xa $za $mx $my $mz] > 2} {draw_line_centers $xa $xa $za $mx $my $mz}
	if {[get_distance $xe $xe $ze $mx $my $mz] > 2} {draw_line_centers $mx $my $mz $xe $ye $ze}

}

#---- draw_line ----
#
proc draw_line {xa ya za xe ye ze} {
	global gvar
	
	# Anfangspunkt der Linie zeichen...
	draw_point $xa $ya $za

	# Anfangs- und Endpunkt sind gleich, dann ist hier Schluss
	if {($xa == $xe) && ($ya == $ye) && ($za == $ze)} {return}

	# Endpunkt der Linie zeichen...
	draw_point $xe $ye $ze
	
	draw_line_centers $xa $ya $za $xe $ye $ze

}


#****************************************************************
# Hier ist kommt das Ablaufprogramm fuer die LEDs...
#
#
#----- led_test -----
#
proc led_test {ebene} {
	global gvar
	draw_line 0 0 0 7 7 7 

	send_cmd [list set_led $gvar(led)]
}




#****** hier geht es los! *******
#
#

# TCP/IP-Socket zum Server oeffnen
set gvar(sock) [socket $gvar(host) $gvar(port)]
fconfigure $gvar(sock) -buffering line
fileevent  $gvar(sock) readable [list receive $gvar(sock)]


# Dimmension des Cubes vom Server holen
send_cmd get_xyz 

# warten, bis Replay vom Server kam und Ergebnis verarbeitet wurde
vwait gvar(led_count)

# die eigentliche Ablaufsteuerung fuer die LEDs
led_test 0


vwait forever 
