#*******************************************************************************
#
# cube_viewer.tcl
#
# Autor        : Uwe Berger; 2006, 2007
# Idee         : Mark B. Stucky (http://wiki.tcl.tk/15032)
#
# Beschreibung : Anzeige-Server fuer einen 3D-Cube (beliebiger Kantenlaenge.
#                Der 3D-Cube kann in alle Richtungen gedreht werden.
#                Die Kommunikation mit den Clients erfolgt via TCP/IP-Sockets.
#
#                Von der Sache her, ist dieser Server die "rechentechnische" 
#                Umsetzung des noch zu bauenden 3D-LED-Cube
#
#
#
#********************************************************************************

 # Initialize the user interface
 #
 proc Init { } {
     global gvar

     wm title . "cube_viewer"


     set gvar(server_port) 12345

     set screenWidth  [winfo screenwidth  .]
     set screenHeight [winfo screenheight .]

     set sc  [min $screenWidth $screenHeight]
     set scx [expr {$sc * 0.7}]
     set scy $scx

     set gvar(Xscreen) $scx
     set gvar(Yscreen) $scy

     set gvar(rho)   5.
     set gvar(theta) 0.0
     set gvar(phi)   0.0
     set gvar(dist)  1000.0

     set gvar(x,translate)   0.0
     set gvar(y,translate)   0.0

     set gvar(line_color)    darkgray
     set gvar(led_on_color)  yellow
     set gvar(led_radius)    10

     set gvar(dot_draw)     1
     set gvar(lines_draw)   1
     set gvar(daxis)        1

     set gvar(look_at) [list 0.0 0.0 0.0]

     set pi    [expr {4.0 * atan(1.0)}]
     set mpi   [expr {-1.0 * $pi}]

     set gvar(pi)     $pi
     set gvar(mpi)    $mpi
     set gvar(twopi)  [expr {2.0 * $pi}]

     set bg \#4f4f4f
     set fg grey

     # build up the list of "known objects" to display. (These are
     # connected to buttons on the interface)
     #
     set gvar(obj_list) [list cube from_File]

     #----- Ausgabeflaeche der Grafik ------
     #
     canvas .c -width $scx -height $scy -bg black -bd 0
     pack .c -side left -fill both -expand 1
     update

     set gvar(canv) .c

     bind .c <ButtonPress-1>   {canvas_rotate_start  %W %x %y}
     bind .c <B1-Motion>       {canvas_rotate_drag   %W %x %y}
     bind .c <ButtonRelease-1> {canvas_rotate_end    %W %x %y}

     bind .c <ButtonPress-2>   {canvas_pan_start   %W %x %y}
     bind .c <B2-Motion>       {canvas_pan_drag    %W %x %y}
     bind .c <ButtonRelease-2> {canvas_pan_end     %W %x %y}

     bind .c <ButtonPress-3>   {canvas_zoom_start  %W %x %y}
     bind .c <B3-Motion>       {canvas_zoom_drag   %W %x %y}
     bind .c <ButtonRelease-3> {canvas_zoom_end    %W %x %y}

     bind .c <Configure> {canvas_resize %w %h}


     #----- Bedienelemente -----
     #
     labelframe .f -text "Controls" -padx 4 -pady 4
     pack .f -side right -fill both


     labelframe .f.f0 -text "Select Data" -padx 4 -pady 4 -height 100

     foreach obj $gvar(obj_list) {
         button .f.f0.$obj -text $obj -command "ReadData $obj; redraw" -width 10
         pack .f.f0.$obj -side top
     }

     #---- debug ----
     button .f.f0.debug -text "debug gvar" -command print_gvar -width 10
     pack   .f.f0.debug -side top

     labelframe .f.f1 -text "Camera Position" -padx 4 -pady 4 -height 100

     label .f.f1.lrho  -text "rho"
     label .f.f1.lthet -text "theta"
     label .f.f1.lphi  -text "phi"
     label .f.f1.ldist -text "distance"
     scale .f.f1.rho  -from 1.0   -to 100.0   -orient horizontal -resolution 1.0  \
         -variable gvar(rho)   -orient vertical -command set_rho
     scale .f.f1.dist -from 10.0  -to 20000.0 -orient horizontal -resolution 1.0  \
         -variable gvar(dist)  -orient vertical -command set_dist

     set gvar(dtheta) [dial::dial .f.f1.dthet 50 "Azimuth (z)"   $bg $fg {get_theta } ]
     set gvar(dazmth) [dial::dial .f.f1.dphi  50 "Elevation (x)" $bg $fg {get_phi   } ]

     grid .f.f1.rho     -row 0 -column 0
     grid .f.f1.lrho    -row 2 -column 0
     grid $gvar(dtheta) -row 0 -column 1
     grid $gvar(dazmth) -row 0 -column 2
     grid .f.f1.dist    -row 0 -column 3
     grid .f.f1.ldist   -row 2 -column 3


     labelframe .f.f3 -text "Viewing Options" -padx 4 -pady 4 -height 100
     checkbutton .f.f3.dot_draw -text "Display Dots"     -variable gvar(dot_draw)   -command redraw
     checkbutton .f.f3.lines_draw -text "Display Lines"  -variable gvar(lines_draw) -command redraw
     checkbutton .f.f3.daxis -text "Display Axis"        -variable gvar(daxis)      -command redraw

     grid .f.f3.dot_draw   -row 0 -column 0  -sticky w
     grid .f.f3.lines_draw -row 1 -column 0  -sticky w
     grid .f.f3.daxis      -row 2 -column 0  -sticky w
     
     labelframe .f.f4 -text "Messages" -padx 4 -pady 4 -height 200
     set gvar(msgb) [text .f.f4.msg -relief groove -bd 2 -width 40 -wrap word -yscrollcommand ".f.f4.scroll set"]
     scrollbar .f.f4.scroll -command ".f.f4.msg yview"
     pack .f.f4.scroll -side right -fill y
     pack .f.f4.msg -side top -anchor n 
     
     pack .f.f0 -side top -fill both
     pack .f.f1 -side top -fill both
     pack .f.f3 -side top -fill both
     pack .f.f4 -side top -fill both

 }


 #----- canvas_resize -----
 #
 proc canvas_resize {x y} {
     global gvar

     set screenWidth  $x
     set screenHeight $y
     set sc  [min $screenWidth $screenHeight]
     set scx [expr {$sc * 1.0}]
     set scy $scx
     set gvar(Xscreen) $scx
     set gvar(Yscreen) $scy

     redraw
 }

 #---- fuer debugging: print_var ----
 #
 proc print_gvar {} {
     global gvar
     puts $gvar(points)
     puts $gvar(sort_points)
     puts $gvar(led)
 }


 # The user changed the value of "rho" via the scale...
 # Update the display.
 #
 proc set_rho {val} {
     global gvar

     set gvar(rho) $val
     redraw
 }

 # The user changed the value of "dist" via the scale...
 # Update the display.
 #
 proc set_dist {val} {
     global gvar

     set gvar(dist) $val
     redraw
 }

 #
 # Return the min of two values
 #
 proc min { val1 val2 } {
     if { $val1 <= $val2 } {
         return $val1
     } else {
         return $val2
     }
 }

 #
 # Return the max of two values
 #
 proc max { val1 val2 } {
     if { $val1 >= $val2 } {
         return $val1
     } else {
         return $val2
     }
 }


 #---- MatrixVectorProduct ----
 #
 proc MatrixVectorProduct {M V} {
     set x [lindex $V 0]
     set y [lindex $V 1]
     set z [lindex $V 2]
     set w [lindex $V 3]
     return [list \
        [expr {[lindex $M 0 0]*$x+[lindex $M 1 0]*$y+[lindex $M 2 0]*$z+[lindex $M 3 0]*$w}] \
        [expr {[lindex $M 0 1]*$x+[lindex $M 1 1]*$y+[lindex $M 2 1]*$z+[lindex $M 3 1]*$w}] \
        [expr {[lindex $M 0 2]*$x+[lindex $M 1 2]*$y+[lindex $M 2 2]*$z+[lindex $M 3 2]*$w}] \
        [expr {[lindex $M 0 3]*$x+[lindex $M 1 3]*$y+[lindex $M 2 3]*$z+[lindex $M 3 3]*$w}] ]
 }

 #---- d_move ----
 #
 proc d_move { x y z w } {
     global gvar

     set tmp [MatrixVectorProduct $gvar(Curnt_Trans) [list $x $y $z $w]]

     foreach {xt yt zt wt} $tmp {break}

     if {$zt == 0.0} {
         set zt 0.000001
     }

     set x1 [expr {($gvar(Xscreen)/2.) + round($gvar(dist)*($xt/$zt)) }]
     set y1 [expr {($gvar(Yscreen)/2.) - round($gvar(dist)*($yt/$zt)) }]

     set x1 [expr {$x1 + $gvar(x,translate) }]
     set y1 [expr {$y1 - $gvar(y,translate) }]

     set gvar(Curnt_X_position) $x1
     set gvar(Curnt_Y_position) $y1

     return [list $x1 $y1]
 }

 #---- d_draw -----
 #
 proc d_draw {x y z w {color white} {arrow none} } {
     global gvar

     set tmp [MatrixVectorProduct $gvar(Curnt_Trans) [list $x $y $z $w]]

     foreach {xt yt zt} $tmp {break}

     if {$zt == 0.0} {
         set zt 0.000001
     }

     set x2 [expr {($gvar(Xscreen)/2.) + round($gvar(dist)*($xt/$zt)) }]
     set y2 [expr {($gvar(Yscreen)/2.) - round($gvar(dist)*($yt/$zt)) }]

     set x2 [expr {$x2 + $gvar(x,translate) }]
     set y2 [expr {$y2 - $gvar(y,translate) }]

     $gvar(canv) create line [list $gvar(Curnt_X_position) $gvar(Curnt_Y_position) $x2 $y2] \
         -fill $color -arrow $arrow

     set gvar(Curnt_X_position) $x2
     set gvar(Curnt_Y_position) $y2


     return [list $x2 $y2]
 }


 #---- Set_Viewing_Transform ----
 #
 proc Set_Viewing_Transform { rho theta phi dist } {
     global gvar

     set sinth [expr {sin($theta)}]
     set costh [expr {cos($theta)}]
     set sinph [expr {sin($phi)}]
     set cosph [expr {cos($phi)}]

     set Ct {{0. 0. 0. 0.} {0. 0. 0. 0.} {0. 0. 0. 0.} {0. 0. 0. 0.}}

     lset Ct 0 0 [expr {-$sinth}]
     lset Ct 0 1 [expr {-$costh * $cosph}]
     lset Ct 0 2 [expr {-$costh * $sinph}]
     lset Ct 0 3 0.0

     lset Ct 1 0 [expr {$costh}]
     lset Ct 1 1 [expr {-$sinth * $cosph}]
     lset Ct 1 2 [expr {-$sinth * $sinph}]
     lset Ct 1 3 0.0

     lset Ct 2 0 0.0
     lset Ct 2 1 $sinph
     lset Ct 2 2 [expr {-$cosph}]
     lset Ct 2 3 0.0

     lset Ct 3 0 0.0
     lset Ct 3 1 0.0
     lset Ct 3 2 $rho
     lset Ct 3 3 1.0

     set gvar(Curnt_Trans) $Ct

     set eye_x [expr {$rho * $sinph * $costh}]
     set eye_y [expr {$rho * $sinph * $sinth}]
     set eye_z [expr {$rho * $cosph}]

     set gvar(eye_point) [list $eye_x $eye_y $eye_z]
 }


 
 #---- ReadData ----
 #
 proc ReadData {obj} {
     global gvar

     set gvar(display) "none"

     switch $obj {

         cube {
	     set lvtx {{-0.7 -0.7 -0.7} {-0.7  0.7 -0.7} { 0.7 -0.7 -0.7} { 0.7  0.7 -0.7}
                       {-0.7 -0.7  0.7} {-0.7  0.7  0.7} { 0.7 -0.7  0.7} { 0.7  0.7  0.7}}
             set lcnx {{0 1} {2 3} {4 5} {6 7} {0 2} {1 3} {4 6} {5 7} {0 4} {1 5} {2 6} {3 7}}
             set obj  [list $lvtx $lcnx]
	     set ldx 2
	     set ldy 2
	     set ldz 2
         }

         default {

             # Load data from a file
             set fName [tk_getOpenFile -parent . -title "Select a file to load" \
                            -initialdir "." ]
            if {[file exists $fName]} {
                source $fName
                set obj  [list $lvtx $lcnx]
            } else {
                return
            }
         }
     }


     set gvar(points) [list $lvtx]
     set gvar(lines) [list $lcnx]
     
     set gvar(dx) $ldx
     set gvar(dy) $ldy
     set gvar(dz) $ldz

     # LED-Feld initialisieren (fuer on/off)
     set gvar(led) {}
     for {set i 0} {$i < [llength [lindex $gvar(points) 0]]} {incr i} {set gvar(led) [linsert $gvar(led) end 0] }
 }


 #---- get_distance ----
 #
 proc get_distance {x y z} {
     global gvar

     foreach {xe ye ze} $gvar(eye_point) {break}
     set dist [expr sqrt([expr pow([expr $xe - $x], 2)\
                             + pow([expr $ye - $y], 2)\
			     + pow([expr $ze - $z], 2)])]
     return $dist
 }


 #---- transform_point ----
 #
 proc transform_point {x y z w} {
     global gvar

     set tmp [MatrixVectorProduct $gvar(Curnt_Trans) [list $x $y $z $w]]
     foreach {xt yt zt wt} $tmp {break}
     if {$zt == 0.0} {set zt 0.000001}
     set x1 [expr {($gvar(Xscreen)/2.) + round($gvar(dist)*($xt/$zt)) }]
     set y1 [expr {($gvar(Yscreen)/2.) - round($gvar(dist)*($yt/$zt)) }]
     set x1 [expr {$x1 + $gvar(x,translate) }]
     set y1 [expr {$y1 - $gvar(y,translate) }]
     return [list $x1 $y1]
 }


 #---- line_draw ----
 #
 proc lines_draw { x1 y1 z1 x2 y2 z2 w } {
     global gvar

     foreach {xp1 yp1} [transform_point $x1 $y1 $z1 $w] {break}
     foreach {xp2 yp2} [transform_point $x2 $y2 $z2 $w] {break}

     $gvar(canv) create line [list $xp1 $yp1 $xp2 $yp2] -fill $gvar(line_color) -arrow none

     return [list $xp1 $yp1 $xp2 $yp2]
 }



 #----------------- sort_points -----------------------------
 #
 # Sortiert die zu zeichnenden Punkte in der Reihenfolge
 # ihrer Enfernung vom Betrachtungsstandpunkt (eye_point).
 # Spaeter werden die am weitestens entferneten Punkte
 # zuerst gezeichnet. Bei der Gelegenheit werden dann die
 # restlichen Daten zum Zeichnen der Punkte zusammengestellt.
 # Es werden nur die Punkte betrachtet, die auch wirklich 
 # dargestellt werden sollen (gvar(led))
 #
 proc sort_points {} {
     global gvar

     set i 0
     set gvar(sort_points) {}
     foreach koord [lindex $gvar(points) 0] {
         foreach {x y z} $koord {
             #--- ... nur fuer die Pkte. berechnen, die auch angezeigt werden sollen
	     if {[lindex $gvar(led) $i]} {
	         #--- Distanz berechnen... ---
                 set d [get_distance $x $y $z]
	         #--- ... wenn wir schon dabei sind, gleich die xy-Koord. im Bild bestimmen... ---
                 foreach {xt yt} [transform_point $x $y $z 1] {break}
	         #--- --- und alles in eine Liste packen... ---
	         set gvar(sort_points) [linsert $gvar(sort_points) end [list $d $i $xt $yt $x $y $z]]
	     }
             incr i
         }
     }
     #--- ... und Punkte sortieren (groesste Entfernung zuerst)! ---
     set gvar(sort_points) [lsort -index 0 -decreasing $gvar(sort_points)]
 }


 #---- dot_draw ----
 #
 proc dot_draw {d i xt yt x y z} {
     global gvar

     set r $gvar(led_radius)

     #---- Groesse des Punktes berechnen ---
     # ... r sind 100%, relativ zu rho...
     set r [expr round([expr $gvar(rho) / $d * $r])]

         set item [$gvar(canv) create oval  [expr $xt + $r] [expr $yt + $r] [expr $xt - $r] [expr $yt - $r]\
             -width 1 -outline black -fill $gvar(led_on_color)]
     return [list $xt $yt]
 }


 

 # ---- redraw ----
 #
 proc redraw {args} {
     global gvar

     $gvar(canv) delete all
     Set_Viewing_Transform $gvar(rho) $gvar(theta) $gvar(phi) $gvar(dist)

     #--- Alle Punkte zeichnen ----
     if {$gvar(dot_draw)} {
          sort_points
	  foreach point $gvar(sort_points) {
	      foreach {d i xt yt x y z item} $point {dot_draw $d $i $xt $yt $x $y $z}
	  }
	  set gvar(first_draw) 0
     }

     #--- Verbindungslinien zeichen ---
     if {$gvar(lines_draw)} {
          set koord [lindex $gvar(points) 0]
	  set points [lindex $gvar(lines) 0]
	  foreach {l} $points {
	      lines_draw [lindex [lindex $koord [lindex $l 0]] 0]\
	                 [lindex [lindex $koord [lindex $l 0]] 1]\
	                 [lindex [lindex $koord [lindex $l 0]] 2]\
	                 [lindex [lindex $koord [lindex $l 1]] 0]\
	                 [lindex [lindex $koord [lindex $l 1]] 1]\
	                 [lindex [lindex $koord [lindex $l 1]] 2]\
	      	         1
	  }
     }

     #--- Koordinatensystem zeichnen ----
     if {$gvar(daxis)} {
         draw_axis $gvar(look_at)
     }

 }


  proc canvas_rotate_start {w x y} {
     global gvar

     set gvar(canvas,rotate,coords) [list $x $y]
 }

 proc canvas_rotate_drag {w x y} {
     global gvar

     foreach {newtheta newphi} [canvas_rotate_calc $x $y] {break}

     dial::setValue $gvar(dtheta) $newtheta
     dial::setValue $gvar(dazmth) $newphi

     set gvar(theta) $newtheta
     set gvar(phi)   $newphi

     set gvar(canvas,rotate,coords) [list $x $y]
     redraw
 }

 proc canvas_rotate_end {w x y} {
     global gvar

     foreach {newtheta newphi} [canvas_rotate_calc $x $y] {break}

     dial::setValue $gvar(dtheta) $newtheta
     dial::setValue $gvar(dazmth) $newphi

     set gvar(theta) $newtheta
     set gvar(phi)   $newphi

     set gvar(canvas,rotate,coords) [list $x $y]
     redraw
 }

 proc canvas_rotate_calc {x y} {
     global gvar

     foreach {xold yold} $gvar(canvas,rotate,coords) {break}
     set dx [expr { ($x - $xold) * 0.01}]
     set dy [expr {-($y - $yold) * 0.01}]

     set newtheta    [expr {$gvar(theta) - $dx}]
     set newphi      [expr {$gvar(phi)   + $dy}]

     if {$newtheta > $gvar(pi)} {
         set newtheta [expr {$newtheta - $gvar(twopi)}]
     }
     if {$newtheta < $gvar(mpi)} {
         set newtheta [expr {$newtheta + $gvar(twopi)}]
     }
     if {$newphi > $gvar(pi)} {
         set newphi [expr {$newphi - $gvar(twopi)}]
     }
     if {$newphi < $gvar(mpi)} {
         set newphi [expr {$newphi + $gvar(twopi)}]
     }

     set retval [list $newtheta $newphi]
     return $retval
 }

 proc canvas_pan_start {w x y} {
     global gvar

     set gvar(canvas,pan,coords) [list $x $y]
 }

 proc canvas_pan_drag {w x y} {
     global gvar

     foreach {xold yold} $gvar(canvas,pan,coords) {break}

     set dx  [expr { ($x - $xold) * 1.0}]
     set dy  [expr {-($y - $yold) * 1.0}]

     set gvar(x,translate)  [expr {$gvar(x,translate) + $dx}]
     set gvar(y,translate)  [expr {$gvar(y,translate) + $dy}]

     set gvar(canvas,pan,coords) [list $x $y]

     redraw
 }

 proc canvas_pan_end {w x y} {
     global gvar

     foreach {xold yold} $gvar(canvas,pan,coords) {break}

     set dx  [expr { ($x - $xold) * 0.1}]
     set dy  [expr {-($y - $yold) * 0.1}]

     set gvar(x,translate)  [expr {$gvar(x,translate) + $dx}]
     set gvar(y,translate)  [expr {$gvar(y,translate) + $dy}]

     set gvar(canvas,pan,coords) [list $x $y]

     redraw
 }

 proc canvas_zoom_start {w x y} {
     global gvar

     set gvar(canvas,zoom,coords) [list $x $y]
 }

 proc canvas_zoom_drag {w x y} {
     global gvar

     foreach {xold yold} $gvar(canvas,zoom,coords) {break}

     set dy [expr {-($y - $yold)}]

     if {$dy != 0} {
         set gvar(dist) [expr {$gvar(dist) + $dy * 2}]
     }
     set gvar(canvas,zoom,coords) [list $x $y]
     redraw
 }

 proc canvas_zoom_end {w x y} {
     global gvar

     foreach {xold yold} $gvar(canvas,zoom,coords) {break}

     set dy [expr {-($y - $yold)}]

     if {$dy != 0} {
         set gvar(dist) [expr {$gvar(dist) + $dy * 2}]
     }
     set gvar(canvas,zoom,coords) [list $x $y]
     redraw
 }

 
 proc draw_axis { cntr } {
    global gvar

    set xtrans $gvar(x,translate)
    set ytrans $gvar(y,translate)
    set dist   $gvar(dist)
    set rho    $gvar(rho)

    set gvar(x,translate)   [expr {$gvar(Xscreen) * -0.4}]
    set gvar(y,translate)   [expr {$gvar(Yscreen) * -0.4}]
    set gvar(dist)          500.0
    set gvar(rho)           5.0

    Set_Viewing_Transform $gvar(rho) $gvar(theta) $gvar(phi) $gvar(dist)

    foreach {xc yc zc} $cntr {break}
    set xp [expr {$xc + 0.4}]
    set yp [expr {$yc + 0.4}]
    set zp [expr {$zc + 0.4}]

    d_move $xc $yc $zc 1.0
    d_draw $xp $yc $zc 1.0 red last
    d_move $xc $yc $zc 1.0
    d_draw $xc $yp $zc 1.0 green last
    d_move $xc $yc $zc 1.0
    d_draw $xc $yc $zp 1.0 blue last

 #   $gvar(canv) delete axis_text

    set xp [expr {$xc + 0.5}]
    set yp [expr {$yc + 0.5}]
    set zp [expr {$zc + 0.5}]
    set coords [d_move $xp $yc $zc 1.0]

    $gvar(canv) create text $coords -text X  -justify center -tag axis_text -fill red
    set coords [d_move $xc $yp $zc 1.0]

    $gvar(canv) create text $coords -text Y  -justify center -tag axis_text -fill green
    set coords [d_move $xc $yc $zp 1.0]

    $gvar(canv) create text $coords -text Z  -justify center -tag axis_text -fill blue

    set gvar(x,translate)   $xtrans
    set gvar(y,translate)   $ytrans
    set gvar(dist)          $dist
    set gvar(rho)           $rho

    Set_Viewing_Transform $gvar(rho) $gvar(theta) $gvar(phi) $gvar(dist)

 }

 # Get the value of "theta" from the "dial" and update the display
 #
#---
 proc get_theta {w} {
     global gvar

     set gvar(theta) [dial::getValue $w]
     redraw
 }

 # Get the value of "phi" from the "dial" and update the display
 #
#---
 proc get_phi {w} {
     global gvar

     set gvar(phi) [dial::getValue $w]
     redraw
 }



 #******************************* D I A L ************************************
 #
 # Quick hack at a simple "dial" widget that handles values
 # from 0 - 360 degrees.
 #
 # To be more generic, this probably needs a lot of work...
 #
 namespace eval dial {
     variable dialInfo

     set pi [expr {4.0 * atan(1.0)}]
     set dialInfo(deg_2_rad)     [expr {$pi / 180.0}]
     set dialInfo(rad_2_deg)     [expr {180.0 / $pi}]

     namespace export dial getValue setValue
 }

 # Create a dial : The dial displays values in degrees, but uses
 #                 an internal value in radians.  NOTE: The radian
 #                 value is returned to the user when dial::getValue
 #                 is called.
 #
 # Params :
 #    w        : Name of the dial
 #    diameter : diameter of the dial
 #    dlabel   : the label used to describe the dial
 #    bg       : background color
 #    fg       : foreground color
 #    cmd      : an optional command to execute when the
 #               dial value changes
 #
 proc dial::dial {w diameter dlabel bg fg {cmd ""} } {
     variable dialInfo

     set dialInfo($w,name)   $w

     set dialInfo($w,frame)  [frame $w -class Dial -relief ridge -bd 2]

     set width  [expr {round($diameter + 10)}]
     set height [expr {round($diameter + 10)}]

     set dialInfo($w,canvas) [canvas $w.c -background $bg \
                                  -width $width -height $height \
                                 -closeenough 2.0]

     set wname $dialInfo($w,canvas)

     set dialInfo($wname,name)       $w
     set dialInfo($w,cmd)            $cmd

     set dialInfo($w,value)      0.0
     set dialInfo($w,value,deg)  [expr {$dialInfo($w,value) * $dialInfo(rad_2_deg)}]

     set dialInfo($w,label)  [label $w.label -text $dlabel]

     set dialInfo($w,lvalue) [label $w.lvalue  \
                                  -textvariable dial::dialInfo($w,value,deg)]

     set rad [expr {$diameter/2}]
     set xc  [expr {$width/2  }]
     set yc  [expr {$height/2 }]

     set x0  [expr {$xc - $rad}]
     set y0  [expr {$yc - $rad}]
     set x1  [expr {$xc + $rad}]
     set y1  [expr {$yc + $rad}]

     set coords [list $x0 $y0 $x1 $y1]

     $w.c create oval $coords -fill $fg

     set x1  [expr {$xc + $rad * cos($dialInfo($w,value)) }]
     set y1  [expr {$yc - $rad * sin($dialInfo($w,value)) }]

     set coords [list $xc $yc $x1 $y1]

     set dialInfo($w,xc)  $xc
     set dialInfo($w,yc)  $yc
     set dialInfo($w,rad) $rad

     $w.c create line $coords -fill red -arrow last -tags dial_arrow

     $w.c bind dial_arrow <ButtonPress-1>   {dial::dial_press  %W %x %y}
     $w.c bind dial_arrow <B1-Motion>       {dial::dial_motion %W %x %y}
     $w.c bind dial_arrow <ButtonRelease-1> {dial::dial_motion %W %x %y}

     pack $w.label  -side top
     pack $w.c      -side top
     pack $w.lvalue -side top

     return $w
 }

 # Mouse button down, start tracking the mouse in order to update the dial
 #
 proc dial::dial_press { widget x y } {
     variable dialInfo

     set w $dialInfo($widget,name)

     set xc $dialInfo($w,xc)
     set yc $dialInfo($w,yc)

     set dx [expr {$x  - $xc}]
     set dy [expr {$yc - $y}]
     set theta [expr {atan2($dy,$dx)}]

     set dialInfo($w,value)     $theta
     set dialInfo($w,value,deg) [format "%6.1f" [expr {$theta * $dialInfo(rad_2_deg)}]]

     set rad $dialInfo($w,rad)
     set x1  [expr {$xc + $rad * cos($theta) }]
     set y1  [expr {$yc - $rad * sin($theta) }]

     set coords [list $xc $yc $x1 $y1]
     set canv   $dialInfo($w,canvas)
     $canv coords dial_arrow $coords

     if {[string length $dialInfo($w,cmd)] > 0} {
         eval $dialInfo($w,cmd) $w
     }
 }

 # Mouse button motion or up, continue tracking the mouse in order to update the dial
 #
 proc dial::dial_motion { widget x y } {
     variable dialInfo

     set w $dialInfo($widget,name)

     set xc $dialInfo($w,xc)
     set yc $dialInfo($w,yc)

     set dx [expr {$x  - $xc}]
     set dy [expr {$yc - $y}]
     set theta [expr {atan2($dy,$dx)}]

     set dialInfo($w,value)     $theta
     set dialInfo($w,value,deg) [format %6.1f [expr {$theta * $dialInfo(rad_2_deg)}]]

     set rad $dialInfo($w,rad)
     set x1  [expr {$xc + $rad * cos($theta) }]
     set y1  [expr {$yc - $rad * sin($theta) }]

     set coords [list $xc $yc $x1 $y1]
     set canv   $dialInfo($w,canvas)
     $canv coords dial_arrow $coords

     if {[string length $dialInfo($w,cmd)] > 0} {
         eval $dialInfo($w,cmd) $w
     }
 }

 # The "value" of the dial has changed, so update the arrow on
 # the canvas
 #
 proc dial::update {widget theta} {
     variable dialInfo

     set w  $dialInfo($widget,name)

     set xc  $dialInfo($w,xc)
     set yc  $dialInfo($w,yc)
     set rad $dialInfo($w,rad)

     set dialInfo($w,value)     $theta
     set dialInfo($w,value,deg) [format "%6.1f" [expr {$theta * $dialInfo(rad_2_deg)}]]

     set x1  [expr {$xc + $rad * cos($theta) }]
     set y1  [expr {$yc - $rad * sin($theta) }]

     set coords [list $xc $yc $x1 $y1]
     set canv   $dialInfo($w,canvas)
     $canv coords dial_arrow $coords
 }

 # Get the current value of the dial (in RADIANS)
 #
 proc dial::getValue {widget} {
     variable dialInfo

     set w $dialInfo($widget,name)
     return $dialInfo($w,value)
 }

 # Set the current value of the dial (in RADIANS)
 #
 proc dial::setValue {widget val} {
     variable dialInfo

     set w $dialInfo($widget,name)

     set dialInfo($w,value) $val
     set dialInfo($w,value,deg) [expr {$val * $dialInfo(rad_2_deg)}]

     dial::update $widget $dialInfo($w,value)
 }



#************************ Server-Kommunikations-Routinen ***************************
#
# Dieses Script ist ja eigentlich nur ein reines "Ausgabegeraet", also der Server, der
# etwas anzeigen soll. Was er anzeigt, kommt ber die Schnittstelle (hier ein
# TCP/IP-Port.
# Die folgenden Routinen definieren/initialisieren einen entsprechenden Socket 
# sowie die Kommandos, welche zwischen Client und Server ausgetauscht werden knnen
#
# ToDo: Kommandos noch in einen sicheren Interpreter packen.... 
#

#----- accept -----
#
proc accept {sock addr port} {
	fconfigure $sock -buffering line
	fileevent  $sock readable [list receive $sock $addr $port]
	out_msgb "-> Accept connection: $addr/$port\n"

}                                   

#----- receive -----
proc receive {sock addr port} {
	if {[eof $sock] || [catch {gets $sock line}]} {
		out_msgb "-> Close connection: $addr/$port\n"
		close $sock
	} else {
		#puts "-> Receive from connection ($addr/$port/$sock): $line"
		set cmd [lindex $line 0]
		#puts "-> Command receive: $cmd"
		if {$line != ""} {
			eval $line $sock
		}
	}
}


proc out_msgb {txt} {
	global gvar
	$gvar(msgb) insert end $txt
	$gvar(msgb) yview end
}


#-------------------------------------
# HEX-Wert ohne vorangestelltes 0x...!
#
proc hex2bin {hex} {
	set bin ""
	for {set i 0} {$i<[string length $hex]} {incr i} {
		set digit [string range $hex $i $i]	
		switch [string toupper $digit] {
			0 {append bin "0000"}
			1 {append bin "0001"}
			2 {append bin "0010"}
			3 {append bin "0011"}
			4 {append bin "0100"}
			5 {append bin "0101"}
			6 {append bin "0110"}
			7 {append bin "0111"}
			8 {append bin "1000"}
			9 {append bin "1001"}
			A {append bin "1010"}
			B {append bin "1011"}
			C {append bin "1100"}
			D {append bin "1101"}
			E {append bin "1110"}
			F {append bin "1111"}
		}
	}
	return $bin
}


#***** Serverkommandos *******
#
#
#----- get_xyz -----
#
# Client fordert die Groesse (Kandenlaengen) des Cubes an.
# Es werden die entsprechenden Werte zurueckgesendet
#
proc get_xyz {sock} {
	global gvar
	puts $sock "set_xyz $gvar(dx) $gvar(dy) $gvar(dz)"
	flush $sock
}


#----- set_led -----
#
# Client liefert im Argument den Zustand aller LEDs (on=1; off=0)
# Nach erfolgreichem Setzen und Update der Anzeige wird ein "update_ok"
# zurueckgesendet.
#
proc set_led {led sock} {
	global gvar
#	set gvar(led) [split $led {}]
	set gvar(led) [split [hex2bin "0x$led"] {}]
	redraw
	update
	puts $sock "update_ok"
	flush $sock
	
}

#----- print_message ------
#
# Gibt den, vom Client gesendeten Text in der Message-Box aus
#
proc print_message {txt sock} {
	out_msgb "$txt\n"
}



 #******************************* M A I N ************************************

 proc main { } {
     global gvar

     # Programmfenster initialisieren und zeichnen
     Init
     
     # Cube-Daten lesen/setzen (2x2x2)
     ReadData [lindex $gvar(obj_list) 0]
     
     # einige Variablen berechnen
     Set_Viewing_Transform $gvar(rho) $gvar(theta) $gvar(phi) $gvar(dist)
     
     # TCP/IP-Kommunikationskanal oeffnen und dort lauschen
     set gvar(sock) [socket -server accept $gvar(server_port)]

 }

 main
