/*********************************************
 * vim:sw=8:ts=8:si:et
 * To use the above modeline in vim you must have "set modeline" in your .vimrc
 * Author: Guido Socher
 * Copyright: GPL V2
 * See http://www.gnu.org/licenses/gpl.html
 *
 * functions to convert ntp timestamps into time and date
 *********************************************/

#include "unixtime.h"

PROGMEM static char wday_str[] = "So\0Mo\0Di\0Mi\0Do\0Fr\0Sa";

/************************************************************************
Beschreibung:		monthlen
					isleapyear = 0-1, month=0-11
Globale Variablen:	
Return:				Anzahl Tage im Monat
*************************************************************************/
uint8_t monthlen(uint8_t isleapyear,uint8_t month)
{
	if (month == 1) return (28+isleapyear);
	if (month > 6) month--;
	if (month %2 == 1) return (30);
	return (31);
}


/************************************************************************
Beschreibung:		decode_time
					entschlsselt das NTP-Datum in datetime_t struct 
Globale Variablen:	
Return:				
*************************************************************************/
void decode_time(const uint32_t time_sec, datetime_t *dt)
{
	uint32_t dayclock;
	uint16_t dayno;
	uint8_t summertime;

	dt->year = EPOCH_YR;
	dayclock = (time_sec - GETTIMEOFDAY_TO_NTP_OFFSET) % SECS_DAY;
	dayno = (time_sec - GETTIMEOFDAY_TO_NTP_OFFSET) / SECS_DAY;
	
	dt->second = dayclock % 60UL;
	dt->minute = (dayclock % 3600UL) / 60;
	dt->hour = dayclock / 3600UL;
	dt->wday = (dayno + 4) % 7;			// day 0 was a thursday

	while (dayno >= YEARSIZE(dt->year))
	{
		dayno -= YEARSIZE(dt->year);
		dt->year++;
	}

	dt->month = 0;
	while (dayno >= monthlen(LEAPYEAR(dt->year), dt->month))
	{
		dayno -= monthlen(LEAPYEAR(dt->year), dt->month);
		dt->month++;
	}
	dt->month ++;
	dt->day	= dayno+1;

	// Sommerzeit
	summertime = 1;
	if (dt->month < 3 || dt->month > 10)	// month 1, 2, 11, 12
		summertime = 0;						// -> Winter

	if ((dt->day - dt->wday >= 25) && (dt->wday || dt->hour >= 2))
	{										// after last Sunday 2:00
		if (dt->month == 10)				// October -> Winter
			summertime = 0;
	}
	else
	{										// before last Sunday 2:00
		if (dt->month == 3)				// March -> Winter
			summertime = 0;
	}

	if (summertime)
	{
		dt->hour++;							// add one hour
		if (dt->hour == 24)
		{									// next day
			dt->hour = 0;
			dt->wday++;						// next weekday
			if (dt->wday == 7 )
				dt->wday = 0;
			if (dt->day == monthlen(LEAPYEAR(dt->year), dt->month))
			{								// next month
				dt->day = 0;
				dt->month++;
			}
			dt->day++;
		}
	}
}
